#' Mise en forme en tableau unique des donnees Excel
#'
#' extrait les donnees Excel reparties sur plusieurs tableaux/onglets en un tableau unique vertical
#'
#' @param chemin chemin menant au fichier "remise" à évaluer
#' @param titre nom du fichier remise à évaluer (avec .xlsx à la fin du nom)
#'
#' @return data_flat, un tableau unique des donnees de la remise avec le référencement RxxCxx
#'
#' @examples
#' chemin<-"./exemples_chemin_utilisateur/" # repertoire de travail
#' titre<-"Remise_user_V1.xlsx" # remise a valider
#' data<-MEF_data(chemin,titre)
#' @author Sebastien Gallet <sebastien.gallet@acpr.banque-france.fr>
#' @export


##########################################################
MEF_data<-function(chemin,titre) {
  pkg<-list("tibble",
            "dplyr",
            "tidyr",
            "readxl",
            "stringr",
            "ggplot2")
  fct_gestion_pkg(pkg)
  data("list_periode")
  # declaration de la table de sortie
  data_flat<-as_tibble(t(array(rep(0,5))))
  colnames(data_flat)<-c("nom_participant","onglet","code_ligne","code_col","valeur")
  data_flat$nom_participant<-as.character(data_flat$nom_participant)
  data_flat$onglet<-as.character(data_flat$onglet)
  data_flat$code_ligne<-as.character(data_flat$code_ligne)
  data_flat$code_col<-as.character(data_flat$code_col)

  # donne de depart, attention, la structure actifs est a modifier ici
  data("template_file") # donnees propre a la campagne
  nb_lig<-dim(template_file[1])[1]
  list_onglet_opendata<-paste0(0:2,".Liste des actifs") # liste des onglets des tables ouvertes
  # nom_particip<-read.xlsx( file=paste0(chemin,titre), sheetName = "P.Participant", startRow = 8, endRow=8, colIndex = 3, as.data.frame = FALSE, header = FALSE) #, colClasses = NA, keepFormulas = FALSE, encoding = "unknown", password = NULL, ... )
  nom_particip<-read_xlsx( path=paste0(chemin,titre), sheet = "P.Participant", range = "C8" ,col_names="participant")
  nom_particip<-as.character(nom_particip)

  inc <- 1
  for (i in 1:nb_lig) { # i=3
    onglet<-as.character(template_file[i,1])
    onglet<-Unaccent(onglet)
    if (i>1) {
      if (onglet == Unaccent(as.character(template_file[i-1,1]) ) ) {
        if ( as.character(template_file[i,3])!=as.character(template_file[i-1,3])) {
          j_start <- nb_ligne + j_start
        } else {
          j_start <- 1
        }
        if (as.character(template_file[i,2])!=as.character(template_file[i-1,2])) {
          k_start <- nb_col + k_start
        } else {
          k_start <- 1
        }

      } else {
        j_start <- 1
        k_start <- 1
      }
    } else {
      j_start <- 1
      k_start <- 1
    }

    ligne_deb<-as.numeric(as.character(template_file[i,3]))
    ligne_fin<-as.numeric(as.character(template_file[i,5]))
    col_deb<-as.character(template_file[i,2])
    col_fin<-as.character(template_file[i,4])

    if (nchar(col_deb)>1) {
      col_debnb<-which(LETTERS==substr(col_deb,1,1)) * 26 + which(LETTERS==substr(col_deb,2,2))
    } else {
      col_debnb<-which(LETTERS==substr(col_deb,1,1))
    }
    if (nchar(col_fin)>1) {
      col_finnb<-which(LETTERS==substr(col_fin,1,1)) * 26 + which(LETTERS==substr(col_fin,2,2))
    } else {
      col_finnb<-which(LETTERS==substr(col_fin,1,1))
    }
    nb_col<-(col_finnb-col_debnb+1)

    list_onglet<-excel_sheets(path=paste0(chemin,titre))
    nb_onglet<-which(Unaccent(list_onglet)==onglet)

    tab<-read_xlsx( path=paste0(chemin,titre),
                    sheet = nb_onglet,
                    range =paste0(col_deb,ligne_deb,":",
                                  col_fin,ligne_fin) ,
                    col_names=as.character(1:nb_col)) # ,.name_repair = "universal"

    # cas ou aucune case n'est remplie
    if (dim(tab)[1]==0) {
      nb_lig<-(ligne_fin-ligne_deb+1)
      tab<-array(0,dim=c(nb_lig,nb_col))
     }

    tab<-as.data.frame(tab)

    # gc() # rafraichit la ram
    nb_ligne<-dim(tab)[1]
    nb_col<-dim(tab)[2]

    for  (j in j_start:(j_start + nb_ligne - 1)) {
      if (j < 10) {
        code_ligne <- paste0("R00",(10 * j))
      } else {
        if (j < 100) {
          code_ligne <- paste0("R0" , (10 * j))
        } else {
          code_ligne <- paste0("R" , (10 * j) )
        }
      }

      for ( k in k_start:(k_start + nb_col - 1) ) {
        if (k < 10) {
          code_col <- paste0("C00" , (10 * k) )
        } else {
          code_col <- paste0("C0" , (10 * k) )
        }

        # ecriture dans la table flat
        data_flat$nom_participant[inc]<-nom_particip
        data_flat$onglet[inc]<-onglet
        data_flat$code_ligne[inc]<-as.character(code_ligne)
        data_flat$code_col[inc]<-as.character(code_col)
        data_flat$valeur[inc]<-tab[(j-j_start+1),(k-k_start+1)]
        inc<-inc+1
        data_flat<-add_row(data_flat)
      }}
  }
  data_flat<-data_flat[-dim(data_flat)[1],]

  opendata_flat<-MEF_opendata(chemin,titre,list_periode,list_onglet_opendata,nom_particip)
  data_flat<-rbind(data_flat,opendata_flat)
  # rm("template_file")
  # gc()
  return(data_flat)
}
##########################################################
