#' Generation du catalogue de graphiques de l'actif
#'
#' Generation automatique d'un catalogue de graphiques à partir des donnees de la remise testee
#' L'objectif est de pouvoir faire un control rapide et visuel des donnees en s'assurant qu'elles correspondent bien aux attendus
#'
#' @param data la variable tableau "data" issu de la fonction "MEF_data(chemin,titre)"
#' @param KPI la variable tableau "KPI_log" issu de l'annalyse "rules_checks"
#'
#' @return ecrit les graphiques concernant l'actif au format .png dans un repertoire "graphiques" cree dans "chemin"
#'
#' @examples
#' Avec les variables "KPI_log" chargees dans l'environnement de travail:
#' data<-MEF_data(chemin,titre)
#' fct_graph_actifs(chemin,data)
#'
#' @author Sebastien Gallet <sebastien.gallet@acpr.banque-france.fr>
#' @export


##########################################################

fct_graph_actifs<- function(chemin,data) {

  # exemple a mettre en place
  # ggplot(mpg, aes(displ, hwy)) +
  #   geom_point() +
  #   geom_smooth(color="#69b3a2") +
  #   theme_ipsum()

  # creation dun repertoire graphiques
  chemin_graph<-paste0(chemin,"graphiques")

  # fonction de reconnaissance du nom en fonction de R00xx et C00xx dans le tableau RC_code
  reco<-function(rc) {r<-substr(rc,1,5)
                      c<-substr(rc,6,10)
                      ligne<-which(RC_code$R==r)
                      return(eval(parse(text=paste0("RC_code$",c,"[ligne]"))))
                      }

  data(RC_code)
  list_onglet<-unique(data$onglet)
  colonne_periode<-c("C0060","C0070","C0080","C0090","C0100") #periodes
  type_var<-c("nature" ,"secteur" , "zone geo" , "cote")
  # code de MEF data par colonne actifs
  list_onglet_actifs<-c(18,19,20)
  list_onglet_actifs<-list_onglet[list_onglet_actifs]

  # arrete des messages de warnings
  oldw <- getOption("warn")
  options(warn = -1)
  data$valeur<-as.numeric(as.character(data$valeur))
  options(warn = oldw) # remise a l option par default

  inc<-0
  for (j in 1:4) { # nature / secteur / zone geo / cote  j=1

    for (i in 0:2) { # i=0
      inc<-inc+1
    scenario<-i
    colonne_var<-colnames(RC_code)[j+1]
    nom_colonne<-type_var[j]
    appel_colonne<-paste0(colonne_var,"/",colonne_periode)

    data %>% filter(onglet==list_onglet_actifs[scenario+1],code_col=="C0020/C0060")%>%
      group_by(code_ligne,code_col) %>%
      summarize(n=sum(valeur))

    k=1
    tab<-data %>% filter(onglet==list_onglet_actifs[scenario+1], code_col==appel_colonne[k] ) %>% # 0.actifs , nature
        group_by(code_ligne,code_col) %>% summarize(n=sum(valeur))
    for (k in 2:length(list_periode)) {
      tab<-rbind(tab,data %>% filter(onglet==list_onglet_actifs[scenario+1], code_col==appel_colonne[k] ) %>% # 0.actifs , nature
        group_by(code_ligne,code_col) %>% summarize(n=sum(valeur)))
    }

    tab$periode<-substr(tab$code_col,7,11)
    tab$code_col<-substr(tab$code_col,1,5)

    list<-lapply(paste0(tab$code_ligne,tab$code_col),function(x) reco(x))
    list[which( lapply( list,function(x) length(x)==0)==1 )]<-"non connu"
    tab$code_ligne<-unlist(list)


    tab$periode<-unlist(lapply(tab$periode,function(x) list_periode[which(x==colonne_periode)]))
    tab$code_col<-nom_colonne

    titre<-paste0("Repartition actifs par ",as.character(nom_colonne)," scenario n°",scenario)

    tab<-as.data.frame(tab)
    tab$code_ligne<-as.factor(tab$code_ligne)
    tab$periode<-as.numeric(tab$periode)
    # mise en pourcentage
    tab_graph <- tab  %>%
      group_by( code_ligne,periode,code_col) %>%
      summarise(n = sum(n)) %>%
      group_by(periode) %>%
      mutate(percentage = n / sum(n) *100)
    tab_graph<-as.data.frame(tab_graph)
    # on limite l'affichage a 20 chr
    tab_graph$code_ligne<-substr(tab_graph$code_ligne,1,20)

    p<-ggplot(data=tab_graph,aes(x=periode, y =percentage,fill=code_ligne)) +
      geom_area()+
      # theme_minimal()
      theme_bw() +
      ggtitle(titre) +
      xlab("Annees") + ylab("Part en %") + labs(fill=NULL)

    ggsave(
      filename=paste0(chemin_graph,"/",titre,".png"),
      plot = p,
      device = NULL,
      path = NULL,
      scale = 1,
      width = 5,
      height = 3,
      units = "in")

    # eval(parse(text=paste0(letters[inc],"<-","p")))

  } # fin boucle scenario
} # fin boucle variables nature/zone geo ...

}
