#' Generation du catalogue de graphiques du passif
#'
#' Generation automatique d'un catalogue de graphiques à partir des donnees de la remise testee
#' L'objectif est de pouvoir faire un control rapide et visuel des donnees en s'assurant qu'elles correspondent bien aux attendus
#'
#' @param data la variable tableau "data" issu de la fonction "MEF_data(chemin,titre)"
#' @param KPI la variable tableau "KPI_log" issu de l'annalyse "rules_checks"
#'
#' @return ecrit les graphiques concernant le passif au format .png dans le repertoire "graphiques"
#'
#' @examples
#' Avec les variables "KPI_log" chargees dans l'environnement de travail:
#' data<-MEF_data(chemin,titre)
#' fct_graph(chemin,KPI_log)
#'
#' @author Sebastien Gallet <sebastien.gallet@acpr.banque-france.fr>
#' @export


##########################################################

fct_graph_passif<- function(chemin,KPI) { # KPI<-KPI_log

  # creation dun repertoire graphiques
  chemin_graph<-paste0(chemin,"graphiques")

  list_titre<-unique(KPI$titre)
  list_retenu<-1:length(list_titre)
  list_retenu<-c(13:34)

  KPI$valeur<-as.numeric(as.character(KPI$valeur))
  KPI$periode<-as.numeric(as.character(KPI$periode))

  inc<-0
  for (j in 1:length(list_retenu)) { # j=1
    inc<-inc+1
    titre_graph<-list_titre[list_retenu[j]]
    tab<-KPI %>% filter(titre==titre_graph) %>%
      group_by(onglet,periode) %>%
      summarize(valeur=sum(valeur))
    tab$onglet<-substr(tab$onglet,1,15)

    # graphique
    p<-ggplot(data=tab,aes(x=periode, y =valeur, fill= onglet)) +
      geom_line(aes(color=onglet))+
      # theme_minimal()
      theme_bw() +
      ggtitle(titre_graph) +
      xlab("Annees") + ylab("Valeur") + labs("scenario")

    titre_graph<-str_replace_all(titre_graph, pattern = "/", replacement = "_")

    ggsave(
      filename=paste0(chemin_graph,"/",titre_graph,".png"),
      plot = p,
      device = NULL,
      path = NULL,
      scale = 1,
      width = 5,
      height = 3,
      units = "in")

    # eval(parse(text=paste0(letters[inc],"<-","p")))

  } # fin boucle scenario


}
