#' Application des regles de controle
#'
#' applique les regles de validation au tableau plat .Rda des donnees issues des remises
#'
#' @param data nom du tableau plat au format .Rda des donnees issues de la remise excel a controler
#'
#' @return ecrit les variables erreur_log.Rda (tableau des erreurs) et KPI_log.Rda (tableau des variables d'interets)
#'
#' @examples
#' chemin<-"./exemples_chemin_utilisateur/" # repertoire de travail où se trouve la remise à évaluer
#' titre<-"Remise_user_V1.xlsx" # nom de la remise a évaluer
#' data<-MEF_data(chemin,titre)
#' data("list_periode")
#' rule_checks(chemin,data,list_periode)
#'
#' @author Sebastien Gallet <sebastien.gallet@acpr.banque-france.fr>
#' @export


##########################################################
rule_checks<-function(chemin,data,list_periode){

  data('rules_tab')
  rules_tab$Feuille_A<-Unaccent(rules_tab$Feuille_A)
  rules_tab$Feuille_B<-Unaccent(rules_tab$Feuille_B)

  ## Initialisation du log erreur
  erreur_log<-array("0",dim=c(1,9))
  erreur_log<-as_tibble(erreur_log)
  colnames(erreur_log)<-c("num","titre","organisme","erreur","onglet","col","lign","periode","niveau importance")
  s<-1 #incrementation des lignes de la log erreurs
  KPI_log<-array("",dim=c(1,6))
  KPI_log<-as_tibble(KPI_log)
  colnames(KPI_log)<-c("num","titre","organisme","onglet","periode","valeur")
  KPI_log$valeur<-as.numeric(KPI_log$valeur)
  t<-1 #incrementation des lignes du tableau KPI
  #### boucle sur les lignes de controls ####
  num_test<-length(rules_tab$liste_indice_filtre.A)

  # definition de la liste des organismes
  orga_list<- unique(data %>% dplyr::select("nom_participant"))
  nb_orga_list<-dim(orga_list)[1]

  # boucle sur les organismes: indice w =1
  for (w in 1:nb_orga_list) {
    orga_name<-orga_list[w,1][[1]]

    for (i in 1:num_test) { # boucle sur les lignes de tests i=1
      erreur_log$titre[s] <- rules_tab$titre_check[i]
      niveau_rules<-rules_tab$niveau.importance[i]
      precision<-rules_tab$precision[i]
      type_precision<-rules_tab$Type.de.precision[i]

      # boucle sur les periodes: indice v
      nb_rep_col<-rules_tab$nb.repetition.col.2020.2050[i]
      for (v in 0:(nb_rep_col-1)) { # v=0

        # boucle sur les formules A et B i=1
        if (rules_tab$liste_indice_filtre.B[i]!="") {
          B<-2
        } else {
          B<-1
          resultat_B=""
        }
        for (k in 1:B) {
          if (k==1) { # on considere A
            Feuille<-rules_tab$Feuille_A[i]
            Col<-rules_tab$Col.A[i]
            ind_col<-rules_tab$Ind_col.A[i]
            Lig<-rules_tab$Ligne.A[i]
            eval(parse(text=paste0("Indl=",as.character(rules_tab$liste_indice_filtre.A[i])) ))
            formule<-rules_tab$formule_A[i]
          } else { # formule B
            Feuille<-rules_tab$Feuille_B[i]
            Col<-rules_tab$Col.B[i]
            ind_col<-rules_tab$Ind_col.B[i]
            Lig<-rules_tab$Ligne.B[i]
            if (rules_tab$liste_indice_filtre.B[i]!="") {
              eval(parse(text=paste0("Indl=",as.character(rules_tab$liste_indice_filtre.B[i]))))
            }
            formule<-rules_tab$formule_B[i]
          }
          # mise en forme des indice colonnes
          if ( grepl("/",ind_col) ) { # deux colonnes-> table ouverte, ici des actifs
            a<-regexpr("/",ind_col)
            Indc<-as.character(as.numeric(substr(ind_col,1,(a[[1]]-1))))
            IndVal<-as.character(as.numeric(substr(ind_col,(a[[1]]+1), nchar(ind_col) ))+10*v)
            # formation de l indice colonne C00XX pour les tables ouvertes
            if (nchar(Indc)==2) {Ind<-paste("00",Indc,sep="")} else {
              if (nchar(Indc)==3) {Ind<-paste("0",Indc,sep="")} else {
                if (nchar(Indc)==4) {Ind<-paste("",Indc,sep="")}
              }
            }
            Ind_col<-paste0(Col,Ind,"/") # indice colonne table ouverte
          } else { # une seule colonne-> table fermee
            Ind_col<-""
            IndVal<-as.character(as.numeric(ind_col)+10*v)
          }

          ############################
          # formation de l indice valeur C00XX
          if (nchar(IndVal)==2) {Ind<-paste("00",IndVal,sep="")} else {
            if (nchar(IndVal)==3) {Ind<-paste("0",IndVal,sep="")} else {
              if (nchar(IndVal)==4) {Ind<-paste("",IndVal,sep="")}
            }
          }
          Ind_val<-paste0(Col,Ind) # indice colonne
          Ind_col<-paste0(Ind_col,Ind_val)
          #################################
          # Constitution de la valeur avec formule si necessaire
          # formation de l indice ligne R00YY
          Ind_lig<-as.character(Indl)
          Valeur_ref<-rep(0,length(Ind_lig)) # intitialisation
          for (j in 1:length(Ind_lig)) { # boucle sur different indice de ligne
            if (nchar(Ind_lig[j])==2) {Ind_lig[j]<-paste("00",Ind_lig[j],sep="")} else {
              if (nchar(Ind_lig[j])==3) {Ind_lig[j]<-paste("0",Ind_lig[j],sep="")} else {
                if (nchar(Ind_lig[j])==4) {Ind_lig[j]<-paste("",Ind_lig[j],sep="")}
              }
            }
            Ind_lig[j]<-paste0(Lig,Ind_lig[j]) # indice ligne
            # selection des données correspondant a feuille,colonne,ligne  j=11
            Valeur<- data%>%
              filter(nom_participant==orga_name,onglet==Feuille,code_col==Ind_col,code_ligne==Ind_lig[j]) #
            Valeur_ref[j]<-as.character(Valeur$valeur[1]) # s'il y a plusieur fois le meme nom!

            #######################
            # on enregistre le chiffre trouves dans la variable alphabet

            if (is.na(as.numeric(Valeur_ref[j]))) { # pas de chiffre dans la ref
              eval(parse(text=paste0("a",j,"=0"))) # choix de mettre 0
            } else {
              eval(parse(text=paste0("a",j,"=",Valeur_ref[j])))
            }

          } # fin de la boucle sur j et les chiffres de la formule

          #######################
          # calcul de la formule A ou B dans resultat_int
          if (sum(formule!=c("","_sum_","_ou_","_et_"))==4) { # alors c'est une formule
            eval(parse(text=paste0("resultat_int","=",formule)))
          } else { if (sum(formule!=c("_sum_","_ou_","_et_"))==3) {
            # sans formule
            eval(parse(text=paste0("resultat_int","=a1")))
          } else {
            list_letters<-paste0("c(","a1")
            for (l in 2:length(Ind_lig)) {list_letters<-paste0(list_letters,",a",l)}
            list_letters<-paste0(list_letters,")")

            if (sum(formule!=c("_ou_","_et_"))==2) {
              # somme de ind
              eval(parse(text=paste0("resultat_int","=sum(",list_letters,")")))
            } else {
              # on laisse sous forme de liste
              eval(parse(text=paste0("resultat_int","=",list_letters)))
            }
          }
          }
          if (k==1) {resultat_A<-resultat_int} else {resultat_B<-resultat_int}
        } # fin de la boucle sur A et B

        ###############################
        # type de test
        if (is.na(resultat_A)){
          test<-"ERR"
          erreur_lig<-"formule A"
          phrase_err<-paste0("le membre A du test ",rules_tab$Num.check[i]," est NAN")
        } else {
          resultat_A<-as.numeric(resultat_A)
          if (resultat_B!="") {
            if ((sum( substr(rules_tab$test[i],1,1)==c("=",">","<") )>=1) ) { # test A/B
              precision<-as.numeric(rules_tab$precision[i])
              type_precision<-rules_tab$Type.de.precision[i]
              if (is.na(resultat_B) ){
                test<-"ERR"
                erreur_lig<-"formule B"
                phrase_err<-paste0("le membre B du test ",rules_tab$Num.check[i]," est NAN")
              } else {
                resultat_B<-as.numeric(resultat_B)
                # integration du coef present dans la formule
                eval(parse(text=paste0("resultat_B=",substr(rules_tab$test[i],2,nchar(rules_tab$test[i])),resultat_B) ) )
                if (type_precision=="relative") { # precision relative a A
                  if (resultat_B==0) {
                    test<-"ERR"
                    erreur_lig<-"formule B"
                    phrase_err<-paste0("le membre B de la formule est nul ")
                  } else {
                    calcul<-resultat_A/resultat_B-1
                    if (rules_tab$test[i]=="=") {
                      test<- (calcul>=-precision) && (calcul<=precision)
                    } else {if (rules_tab$test[i]==">") {
                      test<- (calcul>=-precision)
                    } else {
                      test<- (calcul<=precision)
                    }
                    }
                  }
                } else { # precision en absolu
                  calcul<-resultat_A-resultat_B
                  if (rules_tab$test[i]=="=") {
                    test<- (calcul>=-precision) && (calcul<=precision)
                  } else {if (rules_tab$test[i]==">") {
                    test<- (calcul>=-precision)
                  } else {
                    test<- (calcul<=precision)
                  }
                  }

                }

                if (is.na(test)){
                  test<-"ERR"
                  erreur_lig<-"formule test"
                  phrase_err<-paste0("le calcul du test ",rules_tab$Num.check[i]," est NAN")
                } else { # si pas parvenu a calculer = pb
                  # ecriture des resutlats du test
                  if (test==TRUE) {
                    # resultat_A
                  } else {
                    test<-FALSE
                    erreur_lig<-"formule"
                    phrase_err<-paste0("le test ",rules_tab$Num.check[i]," est negatif")
                  }
                }
              }
            }
          } else {
            if ( rules_tab$test[i]==c("_is.na_") ) { # test A uniquement,test empty ou na
              erreur_num<-which(is.na(resultat_A))
              if (length(erreur_num)>0) {
                test<-FALSE
                erreur_lig<-paste(Ind_lig[], collapse = ", ")
                phrase_err<-paste0("les lignes ",erreur_lig," produisent des erreurs")
              } else { test<-TRUE }
            } else { # test A uniquement, application direct de la cellule "test"
              test<-rules_tab$test[i]
              signe<-substr(test,1,1)
              val_sig<-as.numeric(substr(test,2,nchar(test)))
              calcul<-resultat_A
              if (type_precision=="relative") { # precision relative a A
                precision<-as.numeric(precision)*resultat_A } else {precision<-as.numeric(precision) } # precision en absolu
              if ( type_precision!="" ) {
              if ( signe=="=" ) {
                test<- (calcul>=(val_sig-precision)) && (calcul<=(val_sig+precision))
              } else {if (signe==">") {
                test<- (calcul>=(val_sig-precision) )
              } else {
                test<- (calcul<=(val_sigprecision))
              }
              }
              } else {test<- eval(parse(text=paste0(calcul,signe,val_sig)))}
              if (test==TRUE) {
                erreur_lig<-""
                phrase_err<-paste0("le test est passe avec succes")
                # resultat_A
              } else {
                erreur_lig<-"formule"
                phrase_err<-paste0("le test ",rules_tab$Num.check[i]," est negatif")
              }
            }
          }
        }

        ###############################
        # ecriture dans les log erreur
        if (test==FALSE || test=="ERR") {
          erreur_log$num[s]<-s
          erreur_log$titre[s] <- rules_tab$titre_check[i]
          erreur_log$organisme[s]<-orga_name #TBD
          erreur_log$erreur[s]<-phrase_err
          erreur_log$onglet[s]<-rules_tab$Feuille_A[i]
          erreur_log$col[s]<-Ind_col
          erreur_log$lign[s]<-erreur_lig
          erreur_log$periode[s]<-list_periode[v+1]
          erreur_log$`niveau importance` <-niveau_rules
          s<-s+1
          erreur_log<-erreur_log %>% add_row()
        }
        # ecriture dans le log KPI
        if (rules_tab$resultat_KPI[i]=="y" && length(resultat_A)==1) {
          KPI_log$num[t]<-t
          KPI_log$titre[t]<-rules_tab$Titre.KPI[i]
          KPI_log$organisme[t]<-orga_name
          KPI_log$onglet[t]<-rules_tab$Feuille_A[i]
          KPI_log$periode[t]<-list_periode[v+1]
          KPI_log$valeur[t]<-resultat_A
          t<-t+1
          KPI_log<-KPI_log%>% add_row()
        }

      } # repetition sur les periode indice u
    } # fin de la boucle sur i et les differentes lignes de tests
  } # repetition sur les organismes indice w
  erreur_log<-as.data.frame(erreur_log)
  KPI_log<-as.data.frame(KPI_log)
  erreur_log<-erreur_log[-length(erreur_log[,1]),]
  KPI_log<-KPI_log[-length(KPI_log[,1]),]
  save(erreur_log,file = paste0(chemin,"erreur_log.Rda") )
  save(KPI_log,file = paste0(chemin,"KPI_log.Rda") )
  # rm("rules_tab")
  gc()
}
##########################################################
