#' Import excel et Application des regles de controle
#'
#' extrait les donnes Excel reparties sur plusieurs tableaux/onglets en un tableau unique vertical puis realise les tests de validations définis par le package
#' La fonction realise l'ecriture des fichiers resultats en .csv pour une exploitation sous excel
#'
#' @param chemin chemin menant au fichier "remise" à évaluer
#' @param titre nom du fichier remise a évaluer (avec .xlsx à la fin du nom)
#'
#' @return ecrit les variables erreur_log.Rda (tableau des erreurs) et KPI_log.Rda (tableau des variables d'interets)
#'
#' @examples
#' chemin<-"./exemples_chemin_utilisateur/" # repertoire de travail où se trouve la remise à évaluer
#' titre<-"Remise_user_V1.xlsx" # nom de la remise a évaluer
#' checks_remise(chemin,titre)
#'
#' @author Sebastien Gallet <sebastien.gallet@acpr.banque-france.fr>
#' @export


##########################################################
checks_remise<-function(chemin,titre) { # chemin="./"
  # gestion des library
  pkg<-list("tibble",
            "dplyr",
            "tidyr",
            "readxl",
            "ggplot2")
  fct_gestion_pkg(pkg)
  # extraction des donnees excel
  data<-MEF_data(chemin,titre)
  # import liste des periodes
  data("list_periode")
  # application des regles de contrôle
  rule_checks(chemin,data,list_periode)
  # chargement des fichiers resultats des tests
  load(file=paste0(chemin,"erreur_log.Rda"))
  load(file=paste0(chemin,"KPI_log.Rda"))
  # ecriture des fichiers resultats en .csv pour exploitation sous excel chemin="./"
  write.table(x=erreur_log,file=paste0(chemin,"erreur_log.csv"),sep=";", quote = FALSE, eol = "\n", na = "NA", dec = ".", row.names = FALSE, col.names = TRUE,fileEncoding="latin1") #ecriture des log en .csv
  write.table(x=KPI_log,file=paste0(chemin,"KPI_log.csv"),sep=";", quote = FALSE, eol = "\n", na = "NA", dec = ".", row.names = FALSE, col.names = TRUE,fileEncoding="latin1")

  # creation du catalogue de graphiques
  unlink(paste0(chemin,"graphiques"), recursive = TRUE)
  dir.create(paste0(chemin,"graphiques"))
  fct_graph_actifs(chemin,data)
  fct_graph_passif(chemin,KPI_log)

  }
##########################################################
